==> pyCOMPS-2.3.2-py2.py3-none-any.whl: COMPS/Client.py <==
import logging
import re
import requests
import time
import traceback
import urllib3
import COMPS

urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)

logger = logging.getLogger('COMPS.Client')

class Client(object):
    """
    Client object for managing access to COMPS
    """
    __auth_manager = None

    def __init__(self):
        pass

    @classmethod
    def auth_manager(cls):
        """
        Retrieve the AuthManager.

        Must be logged in first in, otherwise this raises a RuntimeError.

        :return: the AuthManager instance
        """
        if not cls.__auth_manager:
            raise RuntimeError('login() is required.')
        return cls.__auth_manager

    @classmethod
    def login(cls, hoststring):
        """
        Log in to the COMPS service.

        The specified COMPS hoststring allows a couple points of flexibility:

        * Secure vs. Unsecure - Specifying the protocol as http or https allows the user to control
          whether the SSL transport is used for requests.  By default, https is used.
        * Port - Specifying a particular port allows the user to control the port to communicate
          over for requests.  By default, the standard port for the chosen protocol is used
          (i.e. 80 for http, 443 for https).

        For example, the following are all valid formats:

        * comps.idmod.org - uses secure https protocol over port 443.
        * http://internal.idmod.org - uses unsecure http protocol over port 80.
        * localhost:54321 - uses secure https protocol over port 54321.

        Calling login() when already logged into a different host is invalid and will raise a RuntimeError.  When
        already logged into the same host, nothing is done and the function returns immediately.

        :param hoststring
        """
        if not hoststring:
            raise RuntimeError('COMPS host required for login')

        tmphoststring = hoststring.rstrip('/')

        match_obj = re.match('(?:(https?)(?:://))?([\w.-]*)(?:(?::)(\d+))?$', tmphoststring)

        if match_obj is None:
            raise RuntimeError('Invalid format for host string: "{0}".  See help for correct usage.'.format(hoststring))

        logger.debug('Parsed login host: {0}'.format(match_obj.groups()))

        protocol = match_obj.group(1)
        host_name = match_obj.group(2)
        port = match_obj.group(3)

        tmphoststring = '{0}://{1}{2}'.format(
                                                protocol if protocol else 'https',
                                                host_name,
                                                ':{0}'.format(str(port)) if port else ''
                                             )

        if cls.__auth_manager is not None:
            oldhoststring = cls.__auth_manager.hoststring

            if oldhoststring != tmphoststring:
                raise RuntimeError('Already logged into host \'{0}\'.  Must logout before logging into a different host.'.format(oldhoststring))

            if cls.__auth_manager.has_auth_token():
                logger.info('Already logged into {0}.  Skipping login.'.format(tmphoststring))
                return
        else:
            cls.__auth_manager = COMPS.AuthManager(tmphoststring)

        cls.__auth_manager.get_auth_token()   # force credentials prompt and caching of auth token

    @classmethod
    def logout(cls):
        """
        Log out of the COMPS service.

        If logged in, this clears any cached credentials and nulls the AuthManager instance.
        """
        if cls.__auth_manager is not None:
            cls.__auth_manager.clear_auth_token()
        cls.__auth_manager = None

    @classmethod
    def post(cls, path, include_comps_auth_token=True, http_err_handle_exceptions=None, **kwargs):
        return cls.request('POST', path, include_comps_auth_token, http_err_handle_exceptions or [], **kwargs)

    @classmethod
    def put(cls, path, include_comps_auth_token=True, http_err_handle_exceptions=None, **kwargs):
        return cls.request('PUT', path, include_comps_auth_token, http_err_handle_exceptions or [], **kwargs)

    @classmethod
    def get(cls, path, include_comps_auth_token=True, http_err_handle_exceptions=None, **kwargs):
        return cls.request('GET', path, include_comps_auth_token, http_err_handle_exceptions or [], **kwargs)

    @classmethod
    def delete(cls, path, include_comps_auth_token=True, http_err_handle_exceptions=None, **kwargs):
        return cls.request('DELETE', path, include_comps_auth_token, http_err_handle_exceptions or [], **kwargs)

    @classmethod
    def request(cls, method, path, include_comps_auth_token=True, http_err_handle_exceptions=None, **kwargs):
        http_err_handle_exceptions = http_err_handle_exceptions or []

        if 'headers' not in kwargs:
            kwargs['headers'] = {}

        if include_comps_auth_token:
            authtoken = cls.__auth_manager.get_auth_token()
            kwargs['headers'][authtoken[0]] = authtoken[1]

        kwargs['headers']['Accept'] = 'application/json'
        if not kwargs.get('files'):
            kwargs['headers']['Content-Type'] = 'application/json'

        max_tries = 10
        retry_delay_ms = 100

        req_url = cls.__build_url(path, method, kwargs.get('params'))

        for i in range(1, max_tries + 1):
            try:
                resp = requests.request(method,
                                        req_url,
                                        **kwargs)

                if i > 1:
                    logger.debug('Succeeded on attempt {0} of {1}'.format(str(i), str(max_tries)))

                break
            except Exception as e:
                logger.debug('Failed attempt {0} of {1}: {2}'.format(str(i), str(max_tries),
                                                                     traceback.format_exception_only(type(e), e)[0][:-1]))

                if i < max_tries and cls.__should_retry(e):
                    time.sleep(i * retry_delay_ms / 1000)
                    logger.debug('Retrying...')
                else:
                    logger.debug('NOT RETRYING')
                    logger.debug(type(e))
                    if hasattr(e.args[0], 'reason'):
                        logger.debug(type(e.args[0].reason))

                    raise

        if 400 <= resp.status_code < 600 and \
                    resp.status_code not in http_err_handle_exceptions:
            # there was an error, let's try to get a good error-message from the response
            cls.raise_err_from_resp(resp)

        return resp

    @classmethod
    def raise_err_from_resp(cls, resp):
        try:
            json_resp = resp.json()
            resp_msg = json_resp['ResponseMessage']
            corr_id = json_resp.get('CorrelationId')
        except Exception as e:
            # couldn't get a good error-message from the response, just log and then exception out
            logger.debug('Invalid response: {0}'.format(resp.content))
            logger.debug('Exception: {0}'.format(e.message))

            resp.raise_for_status()

        raise RuntimeError('{0} {1} - {2}{3}'.format(str(resp.status_code),
                                                     resp.reason,
                                                     resp_msg,
                                                     ' (CorrelationId = {0})'.format(corr_id) if corr_id else ''))

    @classmethod
    def __build_url(cls, path, method, params):
        url = '{0}{1}/{2}'.format(cls.__auth_manager.hoststring,
                                  '/api' if not path.startswith('asset/') and not path.startswith('/asset/') else '',
                                  (path[1:] if path.startswith('/') else path))
        logger.debug('REQUEST -> {0} {1}{2}'.format(method, url, ' ' + str(params) if params else ''))

        return url

    @classmethod
    def __should_retry(cls, e):
        try:
            if isinstance(e, requests.exceptions.ChunkedEncodingError):
                return True
            elif isinstance(e, requests.exceptions.SSLError):
                return True
            elif isinstance(e, requests.exceptions.ConnectionError) and \
                    hasattr(e.args[0], 'reason') and \
                    ( isinstance(e.args[0].reason, urllib3.exceptions.NewConnectionError) or \
                      isinstance(e.args[0].reason, urllib3.exceptions.SSLError) or \
                      isinstance(e.args[0].reason, requests.exceptions.SSLError) ):
                return True
        except Exception:
            pass

        return False

# Need to do main() stuff this way, otherwise, since we're actually in __main__ module,
# the class variables get re-initialized to None
# See: http://stackoverflow.com/questions/16759035/python-variables-discord-across-modules
#
# def main():
#     import os
#     import sys
#     from COMPS import Client
#     from COMPS.Data import Suite, Experiment, Simulation, SimulationFile, Configuration, QueryCriteria, Priority, WorkItem, WorkItemFile
#     from COMPS.Data.WorkItems import BuilderWorkItem
#     from COMPS.Data.Simulation import SimulationState
#     #from COMPS.Data.WorkItems import BuilderWorkItem
#
#     Client.login(sys.argv[1])
#
#     sims = Simulation.get(query_criteria=QueryCriteria().where('state=Created').where('owner=me'))
#
#     print(sims[:5])
#
#     macros = Client.auth_manager().get_environment_macros('my_environment')
#     print(macros)
#
#     wi = WorkItem.get(id='dd33a1b9-e78b-e611-93fd-f0921c167860',
#                       query_criteria=QueryCriteria().select_children('files'))
#     print(wi)
#
#     exps = wi.get_related_experiments()
#
#     print(exps)
#
#     for e in exps:
#         sims = e.get_simulations()
#         print(sims[0:2])
#
#     s = Simulation.get(id='a70f526d-4d6b-e611-93fd-f0921c16b9e4',
#                        query_criteria=QueryCriteria().select_children('files'))
#     print(s)
#
#     # get the names of all files of type "Output" (except DefaultDemo.json)
#     sfilenames = [ sf.file_name for sf in s.files if sf.file_type == 'input' ]
#
#     print('Found output files: ' + str(sfilenames))
#
#     e = Experiment('test creating a new exp')
#     print('id = ' + str(e.id))
#     print(e.name)
#     e.configuration = Configuration(environment_name='my_environment', priority=Priority.Normal)
#     e.save()
#     e.delete()
#
#     tags = {
#         'sometag': 123,
#         'someothertag': [ 1, 2, 3, { 'foo': 321, 'bar': 'aaa'} ]
#     }
#     e.set_tags(tags)
#
#     s = Simulation('test creating a new sim')
#     print('id = ' + str(s.id))
#     print(s.name)
#     s.configuration = Configuration(environment_name='my_environment', priority=Priority.Normal)
#     s.add_file(simulationfile=SimulationFile('config.json', 'input'),
#                file_path='C:\\temp\\COMPS_Testing_Files\\config.json')
#     with open('C:\\temp\\COMPS_Testing_Files\\campaign.json', 'r') as f:
#         s.add_file(simulationfile=SimulationFile('campaign.json', 'input'),
#                    data=f.read())
#     s.save()
#     # s.delete()
#
#     tags = {
#         'sometag': 123,
#         'someothertag': [ 1, 2, 3, { 'foo': 321, 'bar': 'aaa'} ]
#     }
#     s.set_tags(tags)
#
#     s.commission()
#     print(s)
#
#     s2 = Simulation('test creating a new sim')
#     print('id = ' + str(s.id))
#     print(s2.name)
#     s2.configuration = Configuration(environment_name='my_environment')
#     s2.add_file(simulationfile=SimulationFile('config_original.json', 'input'),
#                 file_path='C:\\temp\\COMPS_Testing_Files\\config_original.json')
#
#     # s.save()
#     Simulation.save_all()
#     s.refresh()
#     print('trying overriding __str__')
#     print(str(s))
#
#     s = Simulation.get(id='cb0f526d-4d6b-e611-93fd-f0921c16b9e4', query_criteria=QueryCriteria().select_children(['files','hpc_jobs']))
#     print(s)
#     s.refresh()
#
#     e2 = Experiment.get(id='89025fc4-0190-e611-93fd-f0921c167860', query_criteria=QueryCriteria().select_children(['tags','configuration']))
#     print(str(e2))
#     e2.refresh()
#
#     s2 = Suite.get(id='88025fc4-0190-e611-93fd-f0921c167860', query_criteria=QueryCriteria().select_children('tags'))
#     print(str(s2))
#     s2.refresh()
#
#     bwi = BuilderWorkItem('my wi', 'my_environment')
#
#     tags = {
#         'Demo': '',
#         'WorkItem type': 'Builder demo'
#     }
#
#     bwi.set_tags(tags)
#
#     bwi.add_work_order(file_path='C:\\temp\\COMPS_Testing_Files\\test_workorder.json')
#
#     bwi.add_file(workitemfile=WorkItemFile('config.json', 'CONFIG_TEMPLATE'), file_path='C:\\temp\\COMPS_Testing_Files\\config.json')
#     bwi.add_file(workitemfile=WorkItemFile('campaign.json', 'CAMPAIGN_TEMPLATE'), file_path='C:\\temp\\COMPS_Testing_Files\\campaign.json')
#
#     bwi.save()
#     bwi.refresh()
#     print(bwi)
#
#
# if __name__ == '__main__':
#     import Client
#     main()

==> pyCOMPS-2.3.3-py2.py3-none-any.whl: COMPS/Client.py <==
import os
import logging
import re
import requests
import time
import traceback
import urllib3
import COMPS

urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)

logger = logging.getLogger('COMPS.Client')

class Client(object):
    """
    Client object for managing access to COMPS
    """
    __auth_manager = None

    def __init__(self):
        pass

    @classmethod
    def auth_manager(cls):
        """
        Retrieve the AuthManager.

        Must be logged in first in, otherwise this raises a RuntimeError.

        :return: the AuthManager instance
        """
        if not cls.__auth_manager:
            raise RuntimeError('login() is required.')
        return cls.__auth_manager

    @classmethod
    def login(cls, hoststring):
        """
        Log in to the COMPS service.

        The specified COMPS hoststring allows a couple points of flexibility:

        * Secure vs. Unsecure - Specifying the protocol as http or https allows the user to control
          whether the SSL transport is used for requests.  By default, https is used.
        * Port - Specifying a particular port allows the user to control the port to communicate
          over for requests.  By default, the standard port for the chosen protocol is used
          (i.e. 80 for http, 443 for https).

        For example, the following are all valid formats:

        * comps.idmod.org - uses secure https protocol over port 443.
        * http://internal.idmod.org - uses unsecure http protocol over port 80.
        * localhost:54321 - uses secure https protocol over port 54321.

        Calling login() when already logged into a different host is invalid and will raise a RuntimeError.  When
        already logged into the same host, nothing is done and the function returns immediately.

        :param hoststring: the COMPS host to connect to
        """
        if not hoststring:
            raise RuntimeError('COMPS host required for login')

        tmphoststring = hoststring.rstrip('/')

        match_obj = re.match('(?:(https?)(?:://))?([\w.-]*)(?:(?::)(\d+))?$', tmphoststring)

        if match_obj is None:
            raise RuntimeError('Invalid format for host string: "{0}".  See help for correct usage.'.format(hoststring))

        logger.debug('Parsed login host: {0}'.format(match_obj.groups()))

        protocol = match_obj.group(1)
        host_name = match_obj.group(2)
        port = match_obj.group(3)

        tmphoststring = '{0}://{1}{2}'.format(
                                                protocol if protocol else 'https',
                                                host_name,
                                                ':{0}'.format(str(port)) if port else ''
                                             )

        if cls.__auth_manager is not None:
            oldhoststring = cls.__auth_manager.hoststring

            if oldhoststring != (os.environ.get('COMPS_SERVER') or tmphoststring).rstrip('/'):
                raise RuntimeError('Already logged into host \'{0}\'.  Must logout before logging into a different host.'.format(oldhoststring))

            if cls.__auth_manager.has_auth_token():
                logger.info('Already logged into {0}.  Skipping login.'.format(oldhoststring))
                return
        else:
            cls.__auth_manager = COMPS.AuthManager(tmphoststring)

        cls.__auth_manager.get_auth_token()   # force credentials prompt and caching of auth token

    @classmethod
    def logout(cls):
        """
        Log out of the COMPS service.

        If logged in, this clears any cached credentials and nulls the AuthManager instance.
        """
        if cls.__auth_manager is not None:
            cls.__auth_manager.clear_auth_token()
        cls.__auth_manager = None

    @classmethod
    def post(cls, path, include_comps_auth_token=True, http_err_handle_exceptions=None, **kwargs):
        return cls.request('POST', path, include_comps_auth_token, http_err_handle_exceptions or [], **kwargs)

    @classmethod
    def put(cls, path, include_comps_auth_token=True, http_err_handle_exceptions=None, **kwargs):
        return cls.request('PUT', path, include_comps_auth_token, http_err_handle_exceptions or [], **kwargs)

    @classmethod
    def get(cls, path, include_comps_auth_token=True, http_err_handle_exceptions=None, **kwargs):
        return cls.request('GET', path, include_comps_auth_token, http_err_handle_exceptions or [], **kwargs)

    @classmethod
    def delete(cls, path, include_comps_auth_token=True, http_err_handle_exceptions=None, **kwargs):
        return cls.request('DELETE', path, include_comps_auth_token, http_err_handle_exceptions or [], **kwargs)

    @classmethod
    def request(cls, method, path, include_comps_auth_token=True, http_err_handle_exceptions=None, **kwargs):
        http_err_handle_exceptions = http_err_handle_exceptions or []

        if 'headers' not in kwargs:
            kwargs['headers'] = {}

        if include_comps_auth_token:
            authtoken = cls.__auth_manager.get_auth_token()
            kwargs['headers'][authtoken[0]] = authtoken[1]

        kwargs['headers']['Accept'] = 'application/json'
        if not kwargs.get('files'):
            kwargs['headers']['Content-Type'] = 'application/json'

        max_tries = 10
        retry_delay_ms = 100

        req_url = cls.__build_url(path, method, kwargs.get('params'))

        for i in range(1, max_tries + 1):
            try:
                resp = requests.request(method,
                                        req_url,
                                        **kwargs)

                if i > 1:
                    logger.debug('Succeeded on attempt {0} of {1}'.format(str(i), str(max_tries)))

                break
            except Exception as e:
                logger.debug('Failed attempt {0} of {1}: {2}'.format(str(i), str(max_tries),
                                                                     traceback.format_exception_only(type(e), e)[0][:-1]))

                if i < max_tries and cls.__should_retry(e):
                    time.sleep(i * retry_delay_ms / 1000)
                    logger.debug('Retrying...')
                else:
                    logger.debug('NOT RETRYING')
                    logger.debug(type(e))
                    if hasattr(e.args[0], 'reason'):
                        logger.debug(type(e.args[0].reason))

                    raise

        if 400 <= resp.status_code < 600 and \
                    resp.status_code not in http_err_handle_exceptions:
            # there was an error, let's try to get a good error-message from the response
            cls.raise_err_from_resp(resp)

        return resp

    @classmethod
    def raise_err_from_resp(cls, resp):
        try:
            json_resp = resp.json()
            resp_msg = json_resp['ResponseMessage']
            corr_id = json_resp.get('CorrelationId')
        except Exception as e:
            # couldn't get a good error-message from the response, just log and then exception out
            logger.debug('Invalid response: {0}'.format(resp.content))
            logger.debug('Exception: {0}'.format(str(e)))

            resp.raise_for_status()

        raise RuntimeError('{0} {1} - {2}{3}'.format(str(resp.status_code),
                                                     resp.reason,
                                                     resp_msg,
                                                     ' (CorrelationId = {0})'.format(corr_id) if corr_id else ''))

    @classmethod
    def __build_url(cls, path, method, params):
        url = '{0}{1}/{2}'.format(cls.__auth_manager.hoststring,
                                  '/api' if not path.startswith('asset/') and not path.startswith('/asset/') else '',
                                  (path[1:] if path.startswith('/') else path))
        logger.debug('REQUEST -> {0} {1}{2}'.format(method, url, ' ' + str(params) if params else ''))

        return url

    @classmethod
    def __should_retry(cls, e):
        try:
            if isinstance(e, requests.exceptions.ChunkedEncodingError):
                return True
            elif isinstance(e, requests.exceptions.SSLError):
                return True
            elif isinstance(e, requests.exceptions.ConnectionError) and \
                    hasattr(e.args[0], 'reason') and \
                    ( isinstance(e.args[0].reason, urllib3.exceptions.NewConnectionError) or \
                      isinstance(e.args[0].reason, urllib3.exceptions.SSLError) or \
                      isinstance(e.args[0].reason, requests.exceptions.SSLError) ):
                return True
        except Exception:
            pass

        return False
